use crate::ems::emu::BroadcastMessage;
use serde::{Deserialize, Serialize};
use tokio::join;

pub mod bms;
pub mod emu;
pub mod pcs;
pub mod tms;

pub type Producer = tokio::sync::mpsc::Sender<String>;
pub type Consumer = tokio::sync::broadcast::Receiver<BroadcastMessage>;
#[async_trait::async_trait]
pub trait Service: Send + Sync {
    async fn serve(&self) {
        let read_task = self.south();
        let write_task = self.north();
        let _ = join!(read_task, write_task);
    }

    async fn south(&self);

    async fn north(&self);
}

#[derive(Serialize, Deserialize)]
pub(crate) struct MyPayload {
    pub key: String,
    pub value: Option<serde_json::Value>,
}

///
/// 推送消息的宏
#[macro_export]
macro_rules! publish_data {
    ($client:expr, $vec:expr, $topic:expr, $code_type:path) => {{
        let payload: Vec<_> = $vec
            .iter()
            .filter(|it| it.code == $code_type && it.key.is_some())
            .map(|it| crate::ems::MyPayload {
                key: it.key.clone().unwrap(),
                value: it.json_value(),
            })
            .collect();

        if !payload.is_empty() {
            $client
                .publish(
                    $topic,
                    serde_json::to_string(&payload)?,
                    QoS::AtMostOnce,
                    false,
                )
                .await?;
        }
    }};
}

